@@ -4,7 +4,7 @@
 package Lingua::Interset::FeatureStructure;
 use strict;
 use warnings;
-our $VERSION = '2.046';
+our $VERSION = '2.047';
 
 use utf8;
 use open ':utf8';
@@ -1399,7 +1399,18 @@ sub clear
 
 
 #------------------------------------------------------------------------------
-# Analogically, get() is a generic feature value getter.
+# Returns a list of names of (known) features whose values are not empty.
+#------------------------------------------------------------------------------
+sub get_nonempty_features
+{
+    my $self = shift;
+    return grep {defined($self->{$_}) && $self->{$_} ne ''} known_features();
+}
+
+
+
+#------------------------------------------------------------------------------
+# get() is a generic feature value getter.
 #------------------------------------------------------------------------------
 sub get
 {
@@ -1462,7 +1473,7 @@ sub get_hash
         my $value = $self->get($feature);
         if(defined($value) && $value ne '')
         {
-            $fs{$feature} = $self->get($feature);
+            $fs{$feature} = $value;
         }
     }
     return \%fs;
@@ -1850,6 +1861,29 @@ sub as_string
 
 
 
+#------------------------------------------------------------------------------
+# Generates text from contents of feature structure in the form used in the
+# FEATS column of the CoNLL-X file format. The tagset and other features will
+# be omitted. Multivalues will be separated by a comma because the vertical bar
+# separates features.
+#------------------------------------------------------------------------------
+sub as_string_conllx
+{
+    my $self = shift;
+    my @features = grep {$_ ne 'tagset' && $_ ne 'other' && defined($self->{$_}) && $self->{$_} ne ''} known_features();
+    my @assignments = map
+    {
+        my $f = $_;
+        my $v = $self->get_joined($f);
+        $v =~ s/\|/,/g;
+        "$f=$v";
+    }
+    (@features);
+    return (scalar(@assignments) > 0) ? join('|', @assignments) : '_';
+}
+
+
+
 #------------------------------------------------------------------------------
 # Recursively converts a structure to string describing a Perl constant.
 # Useful for using eval.
@@ -2274,7 +2308,7 @@ Lingua::Interset::FeatureStructure - Definition of morphosyntactic features and
 
 =head1 VERSION
 
-version 2.046
+version 2.047
 
 =head1 SYNOPSIS
 
@@ -2391,6 +2425,15 @@ We can also clear several features at once:
 
   $fs->clear ('pos', 'prontype', 'gender');
 
+=head2 get_nonempty_features()
+
+Returns a list of names of features whose values are not empty.
+
+  my @features = $fs->get_nonempty_features();
+  my @values   = map { $fs->get_joined ($_) } @features;
+
+The features are returned in a pre-defined (but not alphabetical) order.
+
 =head2 get()
 
 A generic getter for any feature. These two statements do the same thing:
@@ -2423,7 +2466,10 @@ Unlike in C<get_joined()>, this method does I<not sort> the list before returnin
 
   my $hashref = $fs->get_hash();
 
-Creates a hash of all features and their values.
+Creates a hash of all non-empty features and their values.
+The values are identical to what the C<get($feature)> method would return;
+in particular, the value may be a reference to an array.
+
 Returns a reference to the hash.
 
 =head2 get_other_for_tagset()
@@ -2679,6 +2725,32 @@ If the expected value is preceded by "!~", then it is a regular expression which
 =head2 as_string()
 
 Generates a textual representation of the feature structure so it can be printed.
+Features are in a predefined (but not alphabetical) order. Complex values of
+the C<other> feature are serialized in depth. If a feature has multiple values,
+they are sorted alphabetically and delimited by the vertical bar character.
+What follows is a sample output for the C<cs::pdt> tags C<NNMS1-----A---->,
+C<Ck-P1----------> and C<VpQW---XR-AA--->:
+
+  [pos="noun", negativeness="pos", gender="masc", animateness="anim", number="sing", case="nom", tagset="cs::pdt"]
+  [pos="adj", numtype="ord", number="plur", case="nom", tagset="cs::pdt", other={"numtype" => "suffix"}]
+  [pos="verb", negativeness="pos", gender="fem|neut", number="plur|sing", verbform="part", tense="past", voice="act", tagset="cs::pdt"]
+
+=head2 as_string_conllx()
+
+Generates a textual representation of the feature structure in the form used in
+the FEATS column of the CoNLL-X file format. The C<tagset> and C<other>
+features are omitted. Features are in predefined (but not alphabetical) order.
+If a feature has multiple values, they are sorted alphabetically and delimited
+by comma (because the vertical bar is used to separate features).
+What follows is a sample output for the C<cs::pdt> tags C<NNMS1-----A---->,
+C<Ck-P1----------> and C<VpQW---XR-AA--->:
+
+  pos=noun|negativeness=pos|gender=masc|animateness=anim|number=sing|case=nom
+  pos=adj|numtype=ord|number=plur|case=nom
+  pos=verb|negativeness=pos|gender=fem,neut|number=plur,sing|verbform=part|tense=past|voice=act
+
+If there the values of all features (including C<pos>) are empty, the method
+returns the underscore character. Thus the result is never undefined or empty.
 
 =head2 enforce_permitted_values()
 
